# Unit tests for the LinkedList class
#
# Author: David Kauchak
# Date: 2/27/2012

require "linked_list"
require "test/unit"

class TestLinkedList < Test::Unit::TestCase

	def setup
		@l = LinkedList.new
	end

	# test adding and removing for consistency
	def test_add_and_remove		
		@l.add(1)
		@l.add(2)
		@l.add(3)
		
		assert_equal(3, @l.remove)
		assert_equal(2, @l.remove)
		assert_equal(1, @l.remove)
	end

	# test adding and removing multiple elements for
	# consistency
	def test_add_and_remove_better
		(1..10).each{|i| @l.add(i)}
		10.downto(1).each{|i| assert_equal(i, @l.remove)}
	end
	
	# make a linked list with three 1s
	def make_111_linked_list
		3.times{|x| @l.add(1)}
	end
		
	# test to_s over a number of additions
	def test_to_s
		1.upto(10).each{|i| @l.add(i)}
		assert_equal((1..10).entries.reverse.join(" "), @l.to_s.strip)
		# or assert_equal("10 9 8 7 6 5 4 3 2 1", l.to_s.strip)
	end
	
	# test intertwined adds and removes
	def test_add_and_remove_intertwined
		make_111_linked_list
		
		@l.remove
		@l.remove
		@l.add(2)
		@l.add(2)
		@l.remove
		@l.add(3)
		@l.add(3)
		@l.remove
		
		assert_equal("3 2 1", @l.to_s.strip)
	end
	
	# test removing from empty linked lists
	def test_remove_empty
		assert_nil(@l.remove, "Should return nil on an empty remove")
		
		@l.add(1)
		@l.remove
		
		assert_nil(@l.remove, "Should return nil on an empty remove")
	end
	
	def test_each_with_index
		max = 10
		1.upto(max).each{|i| @l.add(i)}
		count = 1
		@l.each_with_index do |val, index|
			assert_equal(max+1-count, val)
			assert_equal(count-1, index)
			count += 1
		end
	end
end